@@ -1,6 +1,7 @@
 import os
 import itertools
 from collections import Callable
+from functools import reduce
 
 from django.utils.datastructures import SortedDict
 from django.forms.forms import BaseForm, get_declared_fields, NON_FIELD_ERRORS, pretty_name
@@ -18,7 +19,7 @@ try:
 except ImportError:
     from mongoengine.errors import ValidationError
 from mongoengine.queryset import OperationError, Q
-from mongoengine.connection import _get_db
+from mongoengine.connection import get_db, DEFAULT_CONNECTION_NAME
 from gridfs import GridFS
 
 from .documentoptions import DocumentMetaWrapper
@@ -26,8 +27,8 @@ from .util import with_metaclass, load_field_generator
 
 _fieldgenerator = load_field_generator()
 
-def _get_unique_filename(name):
-    fs = GridFS(_get_db())
+def _get_unique_filename(name, db_alias=DEFAULT_CONNECTION_NAME, collection_name='fs'):
+    fs = GridFS(get_db(db_alias), collection_name)
     file_root, file_ext = os.path.splitext(name)
     count = itertools.count(1)
     while fs.exists(filename=name):
@@ -66,7 +67,6 @@ def _save_iterator_file(field, uploaded_file, file_data=None):
         file_data = field.field.proxy_class(db_alias=field.field.db_alias,
                             collection_name=field.field.collection_name)
     
-    # overwrite an existing file
     if file_data.instance is None:
         file_data.instance = fake_document
         overwrote_instance = True
@@ -78,7 +78,7 @@ def _save_iterator_file(field, uploaded_file, file_data=None):
         file_data.delete()
         
     uploaded_file.seek(0)
-    filename = _get_unique_filename(uploaded_file.name)
+    filename = _get_unique_filename(uploaded_file.name, field.field.db_alias, field.field.collection_name)
     file_data.put(uploaded_file, content_type=uploaded_file.content_type, filename=filename)
     file_data.close()
     
@@ -152,8 +152,11 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
             
             try:
                 upload.file.seek(0)
-                filename = _get_unique_filename(upload.name)
-                field.replace(upload, content_type=upload.content_type, filename=filename)
+                # delete first to get the names right
+                if field.grid_id:
+                    field.delete()
+                filename = _get_unique_filename(upload.name, f.db_alias, f.collection_name)
+                field.put(upload, content_type=upload.content_type, filename=filename)
                 setattr(instance, f.name, field)
             except AttributeError:
                 # file was already uploaded and not changed during edit.
@@ -175,7 +178,9 @@ def save_instance(form, instance, fields=None, fail_message='saved',
     If construct=False, assume ``instance`` has already been constructed and
     just needs to be saved.
     """
-    instance = construct_instance(form, instance, fields, exclude)
+    if construct:
+        instance = construct_instance(form, instance, fields, exclude)
+        
     if form.errors:
         raise ValueError("The %s could not be %s because the data didn't"
                          " validate." % (instance.__class__.__name__, fail_message))
@@ -217,8 +222,7 @@ def document_to_dict(instance, fields=None, exclude=None):
             continue
         if exclude and f.name in exclude:
             continue
-        else:
-            data[f.name] = getattr(instance, f.name)
+        data[f.name] = getattr(instance, f.name, '')
     return data
 
 def fields_for_document(document, fields=None, exclude=None, widgets=None, \
@@ -234,22 +238,17 @@ def fields_for_document(document, fields=None, exclude=None, widgets=None, \
     in the ``fields`` argument.
     """
     field_list = []
-    ignored = []
     if isinstance(field_generator, type):
         field_generator = field_generator()
+        
+    if formfield_callback and not isinstance(formfield_callback, Callable):
+        raise TypeError('formfield_callback must be a function or callable')
     
-    # This is actually a bad way to sort the fields, but the fields keep the order
-    # they were defined on he document (at least with cPython) and I can't see 
-    # any other way for now. Oh, yeah, it works because we sort on the memory address
-    # and hope that the earlier fields have a lower address.
-    sorted_fields = sorted(list(document._fields.values()), key=lambda field: field.__hash__())
-    
-    for f in sorted_fields:
+    for name in document._fields_ordered:
+        f = document._fields.get(name)
         if isinstance(f, ObjectIdField):
             continue
-        if isinstance(f, ListField) and not (hasattr(f.field,'choices') or isinstance(f.field, ReferenceField)):
-            continue
-        if fields is not None and not f.name in fields:
+        if fields and not f.name in fields:
             continue
         if exclude and f.name in exclude:
             continue
@@ -258,25 +257,15 @@ def fields_for_document(document, fields=None, exclude=None, widgets=None, \
         else:
             kwargs = {}
 
-        if formfield_callback is None:
-            formfield = field_generator.generate(f, **kwargs)
-        elif not isinstance(formfield_callback, Callable):
-            raise TypeError('formfield_callback must be a function or callable')
-        else:
+        if formfield_callback:
             formfield = formfield_callback(f, **kwargs)
+        else:
+            formfield = field_generator.generate(f, **kwargs)            
 
         if formfield:
             field_list.append((f.name, formfield))
-        else:
-            ignored.append(f.name)
 
-    field_dict = SortedDict(field_list)
-    if fields:
-        field_dict = SortedDict(
-            [(f, field_dict.get(f)) for f in fields
-                if ((not exclude) or (exclude and f not in exclude)) and (f not in ignored)]
-        )
-    return field_dict
+    return SortedDict(field_list)
 
 
 
@@ -391,23 +380,22 @@ class BaseDocumentForm(BaseForm):
         # Build up a list of fields that should be excluded from model field
         # validation and unique checks.
         for f in self.instance._fields.values():
-            field = f.name
             # Exclude fields that aren't on the form. The developer may be
             # adding these values to the model after form validation.
-            if field not in self.fields:
+            if f.name not in self.fields:
                 exclude.append(f.name)
 
             # Don't perform model validation on fields that were defined
             # manually on the form and excluded via the ModelForm's Meta
             # class. See #12901.
-            elif self._meta.fields and field not in self._meta.fields:
+            elif self._meta.fields and f.name not in self._meta.fields:
                 exclude.append(f.name)
-            elif self._meta.exclude and field in self._meta.exclude:
+            elif self._meta.exclude and f.name in self._meta.exclude:
                 exclude.append(f.name)
 
             # Exclude fields that failed form validation. There's no need for
             # the model fields to validate them as well.
-            elif field in list(self._errors.keys()):
+            elif f.name in list(self._errors.keys()):
                 exclude.append(f.name)
 
             # Exclude empty fields that are not required by the form, if the
@@ -417,7 +405,7 @@ class BaseDocumentForm(BaseForm):
             # value may be included in a unique check, so cannot be excluded
             # from validation.
             else:
-                field_value = self.cleaned_data.get(field, None)
+                field_value = self.cleaned_data.get(f.name, None)
                 if not f.required and field_value in EMPTY_VALUES:
                     exclude.append(f.name)
         return exclude
